'use strict';
var mysql  = require('@gopalroy/mysql-pool');
var Entity = require('@gopalroy/biz-core').Entity;

var m = ApiHelper;

function ApiHelper() {
}

m.extractProps = function (obj, props) {
    var ret = {};
    for (var i = 0; i < props.length; i++) {
        if (obj[props[i]] !== undefined) {
            ret[props[i]] = obj[props[i]];
        }
    }
    return ret;
};

m.extend = function (obj, props) {
    Object.keys(props).forEach(k => {
        obj[k] = props[k];
    });
    return obj;
};

m.excludeProps = function (obj, props) {
    Object.keys(props).forEach(k => {
        delete obj[k];
    });
};

m.makeQueryArgs = function (query) {
    var result = {filter: {}};
    if (query) {
        for (var k in query) {
            if (k === 'sortBy') result.sortBy = query[k];
            else if (k === 'pageSize') result.pageSize = Number(query[k]);
            else if (k === 'nextMarker') result.nextMarker = Number(query[k]);
            else {
                if (query[k] === "null") // null被解释为特别含义
                    result.filter[k] = null;
                else
                    result.filter[k] = this.parseRegExp(query[k]);
            }
        }
    }

    return result;
};

m.parseRegExp = function (strBuffer) {
    if (strBuffer && strBuffer.length > 2 && strBuffer[0] === '/' && strBuffer[strBuffer.length - 1] === '/') {
        return new RegExp(strBuffer.substr(1, strBuffer.length - 2));
    } else {
        return strBuffer;
    }
};

m.batchUpdateG = function*(tableName, filter, values) {
    var sql   = "UPDATE ?? SET ";
    var args  = [tableName];
    var where = Entity.makeWhereClause(filter);

    var columns = yield mysql.pool.queryP('SHOW COLUMNS FROM ??', args);
    var flag    = false;
    for (var i = 0; i < columns.length; i++) {
        if (values[columns[i].Field]) {
            if (flag) sql += ",";
            flag = true;
            sql += "?? = ?";
            args.push(columns[i].Field);
            args.push(values[columns[i].Field]);
        }
    }
    if (flag) {
        if (where.args.length > 0) sql += ' WHERE ' + where.where;
        args = args.concat(where.args);
        yield mysql.pool.queryP(sql, args);
    }
};

module.exports = m;
