'use strict';
var co      = require('co');
var moment  = require('moment');
var Org     = require('@gopalroy/biz-core').Org;
var Staff   = require('@gopalroy/biz-core').Staff;
var User    = require('@gopalroy/biz-eightymiles').User;
var Vehicle = require('@gopalroy/biz-core').Vehicle;

var m = Authority;

function Authority(short_name, fallback) {
    if (Authority[short_name]) {
        return Authority[short_name]; // 同一个4S店的权限验证共享同一个单例
    } else {
        this.short_name = short_name;
        this.fallback   = fallback;
        this.fallback && (this.fallback.short_name = short_name);
        Authority[short_name] = this;
        return this;
    }
}

m.prototype.getS4idG = function*() {
    if (!this.s4id) {
        var result = yield Org.GetOrgsG({short_name: this.short_name}, null, 1);
        if (result.length < 1) {
            if (!this.fallback) throw new Error('没有Org');
            var s4 = Org.FromJson(this.fallback);
            yield s4.addG();
            this.s4id = s4.id;
        } else {
            this.s4id = result[0].id;
        }
    }
    return this.s4id;
};

m.prototype.checkUserToken = function (req, res, next) {
    co(function*() {
        if (!req.cookies.token) {
            res.status(403).json({status: 'failed', errorNo: 'AUTHENTICATE_FAIL', error: '认证失败'});
            return;
        }

        if (!req.incar) req.incar = {};
        try {
            req.incar.loginUser                 = yield User.FromTokenG(req.cookies.token);
            req.incar.loginUser.last_login_time = moment().format('YYYY-MM-DD HH:mm:ss');
            req.incar.loginUser.last_login_src  = req.ip;
            yield req.incar.loginUser.updateG();
            next();
        } catch (ex) {
            res.status(403).json({status: 'failed', errorNo: 'AUTHENTICATE_FAIL', error: '认证失败'});
        }
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.prototype.checkStaffToken = function (req, res, next) {
    co(function*() {
        var token = req.cookies.token; // || 'Dzz4ZeYzysaD+K+xFDTltQ==';
        if (!token) {
            res.status(403).json({status: 'failed', errorNo: 'AUTHENTICATE_FAIL', error: '请先登录'});
            return;
        }

        if (!req.incar) req.incar = {};
        try {
            req.incar.loginStaff                 = yield Staff.FromTokenG(token);
            req.incar.loginStaff.last_login_time = moment().format('YYYY-MM-DD HH:mm:ss');
            req.incar.loginStaff.last_login_ip   = req.id;
            yield req.incar.loginStaff.updateG();
            next();
        } catch (ex) {
            res.status(403).json({status: 'failed', errorNo: 'AUTHENTICATE_FAIL', error: '认证失败'});
        }
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.resolveVehicle = function (req, res, next) {
    co(function*() {
        if (req.incar && req.incar.loginUser) {
            try {
                var vehicle = yield User.GetVehicleG(req.incar.loginUser.id);//from userRouter
                if (vehicle) {
                    req.incar.vehicle = Vehicle.FromJson(vehicle);
                } else {
                    throw new Error('没有车辆');
                }
            } catch (ex) {
                res.json({status: 'failed', errorNo: 'EMPTY_RESULT', error: '系统还没有您的车辆信息'});
                console.log(__filename, ex.stack);
                return;
            }
        } else {
            try {
                req.incar.vehicle = yield Vehicle.FromIdG(req.params.vehicleId);//from staffRouter
            } catch (ex) {
                res.json({status: 'failed', errorNo: 'EMPTY_RESULT', error: '没有此车辆'});
                return;
            }
        }
        next();
    }).catch(function (ex) {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.name + ": " + ex.message});
        console.log(__filename, ex.stack);
    });
};

m.wisecar = new Authority('wisecar', {
    name: '智车', status: 1, biz_mode: 4, prov: '湖北', city: '武汉', address: '武汉关谷软件园C6栋405', phone: '+8618812345678'
});

module.exports = m;
