'use strict';
var co         = require('co');
var express    = require('express');
var User       = require('@gopalroy/biz-eightymiles').User;
var Vehicle    = require('@gopalroy/biz-eightymiles').Vehicle;
var VehicleAPI = require('@gopalroy/eightymiles/api/VehicleAPI');

var m = CareApi;

function CareApi() {
}

m.context = '/';

m.getUserRouter = function () {
    if (!this.userRouter) {
        var router      = new express();
        this.userRouter = router;
        router.get('/care/config', VehicleAPI.value.getCareConfig);
        router.get('/care', this.resolveVehicle, VehicleAPI.value.getCares);//获取保养记录
        router.post('/care', this.resolveVehicle, VehicleAPI.value.addCare);//添加保养记录
        router.put('/care/:careId([0-9]+)', this.resolveVehicle, VehicleAPI.value.updateCare);//修改保养记录
        router.delete('/care/:careId([0-9]+)', this.resolveVehicle, VehicleAPI.value.deleteCare);//删除保养记录
        router.get('/care/status', this.resolveVehicle, VehicleAPI.value.getCareStatus);//获取当前各项保养的最后保养日期
        router.get('/care/advice', this.resolveVehicle, VehicleAPI.value.getCareAdvice);//获取保养建议
    }

    return this.userRouter;
};

m.getStaffRouter = function () {
    if (!this.staffRouter) {
        var router       = new express();
        this.staffRouter = router;
        router.get('/care/config', VehicleAPI.value.getCareConfig);
        router.get('/vehicle/:vehicleId(\\d+)/care', this.resolveVehicle, VehicleAPI.value.getCares);//获取保养记录
        router.post('/vehicle/:vehicleId(\\d+)/care', this.resolveVehicle, VehicleAPI.value.addCare);//添加保养记录
        router.put('/care/:careId([0-9]+)', VehicleAPI.value.updateCare);//修改保养记录
        router.delete('/care/:careId([0-9]+)', VehicleAPI.value.deleteCare);//删除保养记录
        router.get('/vehicle/:vehicleId(\\d+)/care/status', this.resolveVehicle, VehicleAPI.value.getCareStatus);//获取当前各项保养的最后保养日期
        router.get('/vehicle/:vehicleId(\\d+)/care/advice', this.resolveVehicle, VehicleAPI.value.getCareAdvice);//获取保养建议
    }

    return this.staffRouter;
};

m.resolveVehicle = function (req, res, next) {
    co(function*() {
        if (req.incar && req.incar.loginUser) {
            try {
                var vehicle = yield User.GetVehicleG(req.incar.loginUser.id);//from userRouter
                if (vehicle) {
                    req.incar.vehicle = Vehicle.FromJson(vehicle);
                } else {
                    throw new Error('没有车辆');
                }
            } catch (ex) {
                res.json({status: 'failed', errorNo: 'EMPTY_RESULT', error: '系统还没有您的车辆信息'});
                console.log(__filename, ex.stack);
                return;
            }
        } else {
            try {
                req.incar.vehicle = yield Vehicle.FromIdG(req.params.vehicleId);//from staffRouter
            } catch (ex) {
                res.json({status: 'failed', errorNo: 'EMPTY_RESULT', error: '没有此车辆'});
                return;
            }
        }
        next();
    }).catch(function (ex) {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.name + ": " + ex.message});
        console.log(__filename, ex.stack);
    });
};

module.exports = m;

