'use strict';
var co        = require('co');
var core      = require('@gopalroy/biz-core');
var mysql     = require('@gopalroy/mysql-pool');
var express   = require('express');
var Authority = require('./Authority');
var ApiHelper = require('./ApiHelper');

var DetectApi = function () {
};

var m     = DetectApi;
m.context = '/';

m.getUserRouter = function () {
    if (!this.userRouter) {
        var r           = new express();
        this.userRouter = r;
        r.post('/detect', Authority.resolveVehicle, this.detect);
        r.get('/detect', Authority.resolveVehicle, this.getPage);
        r.get('/detect/:id(\\d+)', Authority.resolveVehicle, this.getOne);
    }
    return this.userRouter;
};

m.getStaffRouter = function () {
    if (!this.staffRouter) {
        var r = new express();
        this.staffRouter = r;
        r.get('/vehicle/:vehicleId(\\d+)/detect', Authority.resolveVehicle, this.getPage);
        r.post('/vehicle/:vehicleId(\\d+)/detect', Authority.resolveVehicle, this.detect);
    }

    return this.staffRouter;
};

m.detect = function (req, res) {
    co(function*() {
        var ret = yield req.incar.vehicle.remoteDetectG();
        res.json({status: 'success', data: ret});
    }).catch(ex => {
        if (ex.message) res.json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        else res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.getOne = function (req, res) {
    var id = req.params.id,
        obdCode = req.incar.vehicle.obd_code;
    co(function*() {
        var result = yield mysql.pool.queryP('select * from t_car_detection where id = ? and obdCode = ? limit 1', [id, obdCode]);
        res.json({status: 'success', item: result[0] || null});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

m.getPage = function (req, res) {
    var args    = ApiHelper.makeQueryArgs(req.query),
        vehicle = req.incar.vehicle,
        query   = core.Entity.assemblyPagingSQL('t_car_detection', {obdCode: vehicle.obd_code}, 'createTime-', args.pageSize || 10, args.nextMarker || 1);

    co(function*() {
        var result = yield [mysql.pool.queryP(query.sqlCount, query.args), mysql.pool.queryP(query.sql, query.args)];
        res.json({status: 'success', total: result[0][0].rowsCount, items: result[1]});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

module.exports = m;
