'use strict';
var co         = require('co');
var express    = require('express');
var moment     = require('moment');
var ApiHelper  = require('./ApiHelper');
var Utils      = require('@gopalroy/biz-eightymiles').Utils;
var User       = require('@gopalroy/biz-eightymiles').User;
var authority  = require('./Authority').wisecar;
var VehicleAPI = require('@gopalroy/eightymiles/api/VehicleAPI');

var m = PubApi;

function PubApi() {
}

m.context = '/pub';

m.getPubRouter =
    function () {
        if (!this.pubRouter) {
            var router     = new express();
            this.pubRouter = router;
            router.get('/sendVerifyCodeSms/:phone(\\d+)', this.sendVerifyCodeSms);
            router.get('/checkPhoneNameEmail', this.checkPhoneNameEmail);
            router.post('/register', this.register);
            router.post('/login', this.login);
            router.post('/resetPwd', this.resetPwd);
            router.get('/brand', VehicleAPI.value.getBrands);//获取所有品牌
            router.get('/brand/:brandCode/series', VehicleAPI.value.getSeriesForBrand);//获取指定品牌下的车系清单
        }
        return this.pubRouter;
    };

m.sendVerifyCodeSms =
    function (req, res) {
        var phone = req.params.phone,
            code  = Math.random().toString().substr(2, 6);

        if (!/^\d{11}$/.test(phone)) {
            res.json({status: 'failed', errorNo: 'ILLEGAL_PHONE_NO', error: '手机号格式错误'});
            return;
        }

        if (process.env.WiseCar_ENV === 'TEST') code = '123456';
        co(function*() {
            if (process.env.WiseCar_ENV !== 'TEST') {
                yield Utils.sendSmsCode('智车', phone, code);
            }
            Utils.addCacheData(phone + '-VerifyCode', code, 15 * 60);
            res.json({status: 'success'});
        }).catch(function (ex) {
            res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: '系统异常'});
            console.log(__filename, ex.stack);
        });
    };

m.checkPhoneNameEmail =
    function (req, res) {
        var s4_id, params = req.query, filter, result = {status: 'success'};

        co(function*() {
            s4_id     = yield authority.getS4idG();
            var array = ['name', 'phone', 'email'];
            for (let i in array) {
                if (params[array[i]]) {
                    filter           = {s4_id: s4_id};
                    filter[array[i]] = params[array[i]];
                    result[array[i]] = (yield User.GetUsersG(filter, null, 1, null)).length < 1;
                }
            }
            res.json(result);
        }).catch(function (ex) {
            res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
            console.log(__filename, ex.stack);
        });
    };

m.register =
    function (req, res) {
        var params = ['nick', 'phone', 'pwd', 'code'];
        var user   = User.FromJson(ApiHelper.extractProps(req.body, params));

        if (params.find(e => !user[e])) {
            res.json({status: 'failed', errorNo: 'INCOMPLETE_PARAMS', error: '输入数据不完整'});
        }
        if (!/[0-9]{11}/.test(user.phone)) {
            res.json({status: 'failed', errorNo: 'ILLEGAL_VALUE', error: '请填写正确的手机号，如：18812345678'});
        } else if (!user.pwd || user.pwd.length < 6) {
            res.json({status: 'failed', errorNo: 'ILLEGAL_VALUE', error: '密码至少6位'});
        } else {
            co(function*() {
                if (user.code != (yield Utils.getCacheDataP(user.phone + '-VerifyCode'))) {
                    res.json({status: 'failed', errorNo: 'VERIFY_CODE_MISMATCH', error: '验证码错误'});
                } else {
                    user.s4_id = yield authority.getS4idG();
                    // 验证是否重复注册
                    if (user.name && (yield User.GetUsersG({
                            name : user.name,
                            s4_id: user.s4_id
                        }, null, 1, null)).length > 0) {
                        res.json({status: 'failed', errorNo: 'OCCUPIED_IDENTIFIER', error: '用户名已被占用'});
                    } else if ((yield User.GetUsersG({
                            phone: user.phone,
                            s4_id: user.s4_id
                        }, null, 1, null)).length > 0) {
                        res.json({status: 'failed', errorNo: 'OCCUPIED_IDENTIFIER', error: '手机号已被占用'});
                    } else if (user.email && (yield User.GetUsersG({
                            email: user.email,
                            s4_id: user.s4_id
                        }, null, 1, null)).length > 0) {
                        res.json({status: 'failed', errorNo: 'OCCUPIED_IDENTIFIER', error: '电子邮箱已被占用'});
                    } else {
                        user.pwd             = User.EncryptPassword(user.pwd);
                        user.name            = user.phone;
                        user.tel_pwd         = '';
                        user.last_login_time = moment().format('YYYY-MM-DD HH:mm:ss');
                        user.last_login_src  = req.ip;
                        yield user.addG();
                        res.json({status: 'success', user: user.toSafe()});
                    }
                }
            }).catch(ex => {
                res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
                console.log(__filename, ex.stack);
            });
        }
    };

m.login =
    function (req, res) {
        var params = ['phone', 'pwd'];

        if (params.find(e => !req.body[e])) {
            res.json({status: 'failed', errorNo: 'INCOMPLETE_PARAMS', error: '输入数据不完整'});
            return;
        }

        co(function*() {
            var user = yield User.AuthenticateG(yield authority.getS4idG(), req.body.phone, req.body.pwd);
            if (user) {
                user.last_login_time = moment().format('YYYY-MM-DD HH:mm:ss');
                user.last_login_src  = req.ip;
                yield user.updateG();
                res.cookie('token', user.makeToken());
                res.json({status: 'success', user: user.toSafe()});
            } else {
                res.json({status: "failed", errorNo: 'AUTHENTICATE_FAIL', error: "账号或密码错误"});
            }
        }).catch(ex => {
            res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
            console.log(__filename, ex.stack);
        });
    };

m.resetPwd =
    function (req, res) {
        var params = ['phone', 'code', 'pwd'];
        var reqObj = ApiHelper.extractProps(req.body, params);

        if (params.find(e => !req.body[e])) {
            res.json({status: 'failed', errorNo: 'INCOMPLETE_PARAMS', error: '输入数据不完整'});
            return;
        } else if (!reqObj.pwd || reqObj.pwd.length < 6) {
            res.json({status: 'failed', errorNo: 'ILLEGAL_VALUE', error: '密码至少6位字符'});
            return;
        }

        co(function*() {
            if (reqObj.code !== (yield Utils.getCacheDataP(reqObj.phone + '-VerifyCode'))) {
                res.json({status: 'failed', errorNo: 'VERIFY_CODE_MISMATCH', error: '验证码错误'});
                return;
            }

            var result = yield User.GetUsersG({phone: reqObj.phone, s4_id: yield authority.getS4idG()}, null, 1);
            if (result.length > 0) {
                var user = result[0];
                user.pwd = User.EncryptPassword(reqObj.pwd);
                yield user.updateG();
                res.json({status: 'success'});
            } else {
                res.json({status: 'failed', errorNo: 'EMPTY_RESULT', error: '手机号未注册'});
            }
        }).catch(ex => {
            res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
            console.log(__filename, ex.stack);
        });
    };

module.exports = m;
