'use strict';
var express     = require('express');
var crypto      = require('crypto');
var co          = require('co');
var moment      = require('moment');
var authority   = require('./Authority').wisecar;
var Staff       = require('@gopalroy/biz-core').Staff;
var m           = StaffApi;

function StaffApi() {
}

m.context = '/staff';

m.getStaffRouter = function() {
    if (!this.staffRouter) {
        var r = new express();
        this.staffRouter = r;
        r.post('/login', this.login);
    }

    return this.staffRouter;
};

m.login = function(req, res) {
    var username = req.body.username,
        password = req.body.password,
        cipherAES = crypto.createCipher("aes128", new Buffer(Staff.TokenKey));

    if (!username || !password) {
        return res.json({status: 'failed', errorNo: 'INCOMPLETE_PARAMS', error: '请输入用户名及'});
    }

    co(function*() {
        var s4_id = yield authority.getS4idG();
        var result = yield Staff.GetStaffsG({name: username, s4_id: s4_id});
        if (result.length < 1) {
            return res.json({status: 'failed', errorNo: 'NO_RECORD', error: '没有此用户'});
        } else if (result[0].pwd != crypto.createHash('sha1').update(password).digest('hex')) {
            return res.json({status: 'failed', errorNo: 'PASSWORD_MISMATCH', error: '密码错误'});
        }

        var staff = result[0];
        staff.last_login_time = moment().format('YYYY-MM-DD HH:mm:ss');
        staff.last_login_ip = req.ip;
        yield staff.updateG();
        cipherAES.setAutoPadding(true);
        var token = cipherAES.update(require('util').format("%j", {id: staff.id}), "utf8", "base64");
        token += cipherAES.final("base64");
        res.cookie("token", token);
        res.json({status:"success",staff: staff});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

module.exports = m;
