'use strict';
var co        = require('co');
var express   = require('express');
var moment    = require('moment');
var Authority = require('./Authority');
var mysql     = require('@gopalroy/mysql-pool');
var m         = StatApi;

function StatApi() {
}

m.context = '/';

m.getUserRouter = function () {
    if (!this.userRouter) {
        var r           = new express();
        this.userRouter = r;
        r.get('/stat/report', Authority.resolveVehicle, this.report);
        r.get('/stat/habit', Authority.resolveVehicle, this.drivingHabit);
        r.get('/stat/oil', Authority.resolveVehicle, this.oilUsage);
        r.get('/stat/speed', Authority.resolveVehicle, this.speedDistribution);
    }

    return this.userRouter;
};

m.getStaffRouter = function () {
    if (!this.staffRouter) {
        var router       = new express();
        this.staffRouter = router;
        router.get('/vehicle/:vehicleId(\\d+)/report', Authority.resolveVehicle, this.report);
        router.get('/vehicle/:vehicleId(\\d+)/habit', Authority.resolveVehicle, this.drivingHabit);
        router.get('/vehicle/:vehicleId(\\d+)/oil', Authority.resolveVehicle, this.oilUsage);
        router.get('/vehicle/:vehicleId(\\d+)/speed', Authority.resolveVehicle, this.speedDistribution);
    }

    return this.staffRouter;
};

// 用车报告, 指定时段内用车次数count, 行驶里程mileage, 平均油耗avgOil, 平均车速speed
m.report = function (req, res) {
    var pageSize   = parseInt(req.query.pageSize) || 20;
    var nextMarker = parseInt(req.query.nextMarker) || 1;
    var type       = ['week', 'half-month', 'month', 'season', 'year'].indexOf(req.query.type);
    var sql        = 'select * from t_obd_statistics where s4Id = ? and obdCode = ? and type = ? order by id desc limit ?, ?';
    var sqlCount   = 'select count(*) as total from t_obd_statistics where s4Id = ? and obdCode = ? and type = ?';
    var sqlMax     = 'select max(mileage) as mileage_max, max(avgOil) as avgOil_max, max(`count`) as count_max, ' +
        'max(speed) as speed_max from t_obd_statistics where s4Id = ? and obdCode = ? and type = ?';
    var sqlAllMax  = 'select max(mileage) as mileage_max, max(avgOil) as avgOil_max, max(`count`) as count_max, ' +
        'max(speed) as speed_max from t_obd_statistics where s4Id = ? and type = ?';
    var sqlAllAvg  = 'select avg(mileage) as mileage_avg, avg(avgOil) as avgOil_avg, avg(`count`) as count_avg, ' +
        'avg(speed) as speed_avg from t_obd_statistics where s4Id = ? and type = ?';
    if (type === -1) {
        res.json({
            status : 'failed',
            errorNo: 'REPORT_TYPE_ERROR',
            error  : '请指定报告类型: 周(week), 半月(half-month), 月(month), 季(season), 年(year)'
        });
        return;
    }

    co(function*() {
        var result = yield [mysql.pool.queryP(sqlCount, [req.incar.vehicle.s4_id, req.incar.vehicle.obd_code, type + 1]),
            mysql.pool.queryP(sql, [req.incar.vehicle.s4_id, req.incar.vehicle.obd_code, type + 1, nextMarker - 1, pageSize])];
        var max    = yield mysql.pool.queryP(sqlMax, [req.incar.vehicle.s4_id, req.incar.vehicle.obd_code, type + 1]);
        var allMax = yield mysql.pool.queryP(sqlAllMax, [req.incar.vehicle.s4_id, type + 1]);
        var allAvg = yield mysql.pool.queryP(sqlAllAvg, [req.incar.vehicle.s4_id, type + 1]);
        Object.keys(max[0]).forEach(k => {
            if (max[0][k] === null) max[0][k] = 0;
        });
        res.json({
            status: 'success',
            report: result[1],
            total : result[0][0].total,
            max   : max[0],
            allMax: allMax[0],
            allAvg: allAvg[0]
        });
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

// 驾驶习惯统计,每日急加,急减,急转弯次数
m.drivingHabit = function (req, res) {
    var params  = ['start', 'end'];
    var dateReg = /\d{4}-\d{1,2}-\d{1,2}/;
    var sql     = 'select sum(speedUp) as speedUp, sum(speedDown) as speedDown, sum(sharpTurn) as sharpTurn, ' +
        'STR_TO_DATE(DATE_FORMAT(fireTime, "%Y-%m-%d"), "%Y-%m-%d") AS `date` from t_obd_drive ' +
        'where obdCode = ? and fireTime between ? and ? ' +
        'group by `date` order by `date` desc limit ?';

    if (params.find(e => !req.query[e])) {
        return res.json({status: 'failed', errorNo: 'INCOMPLETE_PARAMS', error: '请输入起止时间'});
    } else if (!dateReg.test(req.query.start) || !dateReg.test(req.query.end)) {
        return res.json({status: 'failed', errorNo: 'ILLEGAL_VALUE', error: '参数格式错误:起止时间参数start,end的格式应为"yyyy-MM-dd"'});
    }

    co(function*() {
        var result       = yield mysql.pool.queryP(sql, [req.incar.vehicle.obd_code, req.query.start, req.query.end, parseInt(req.query.limit) || 30]);
        var totalSpeedUp = 0, totalSpeedDown = 0, totalSharpTurn = 0;
        result.forEach(i => {
            totalSpeedUp += i.speedUp || (i.speedUp = 0);
            totalSpeedDown += i.speedDown || (i.speedDown = 0);
            totalSharpTurn += i.sharpTurn || (i.sharpTurn = 0);
        });
        return res.json({status: 'success', data: result, total: {speedUp: totalSpeedUp, speedDown: totalSpeedDown, sharpTurn: totalSharpTurn}});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

// 油耗统计, 每日行驶里程Km, 油耗L, 平均油耗L/100Km, 及时间段内总里程,总油耗,平均油耗
m.oilUsage = function (req, res) {
    var params  = ['start', 'end'];
    var dateReg = /\d{4}-\d{1,2}-\d{1,2}/;
    var sql     = 'SELECT sum(mileage)/1000 as day_mileage, sum(oil) as day_oil, `date` ' +
        'FROM (SELECT mileage, (currentAvgOilUsed*currentMileage/1000) AS oil, ' +
        '  STR_TO_DATE(DATE_FORMAT(fireTime, \'%Y-%m-%d\'), \'%Y-%m-%d\') AS `date` ' +
        '  FROM t_obd_drive WHERE obdCode = ? AND fireTime BETWEEN ? AND ?) T' +
        '  GROUP BY `date` ORDER BY `date` desc limit ?';

    if (params.find(e => !req.query[e])) {
        return res.json({status: 'failed', errorNo: 'INCOMPLETE_PARAMS', error: '请输入起止时间'});
    } else if (!dateReg.test(req.query.start) || !dateReg.test(req.query.end)) {
        return res.json({
            status : 'failed',
            errorNo: 'ILLEGAL_VALUE',
            error  : '参数格式错误: 起止时间参数start, end的格式应为"yyyy-MM-dd"'
        });
    }

    co(function*() {
        var result        = yield mysql.pool.queryP(sql, [req.incar.vehicle.obd_code, req.query.start, req.query.end, parseInt(req.query.limit) || 30]);
        var total_oil     = 0;
        var total_mileage = 0;

        result.forEach(r => {
            if (!r.day_oil) r.day_oil = 0;
            else total_oil += r.day_oil;

            if (!r.day_mileage) {
                r.day_mileage = 0;
                r.avg_oil = 0;
            } else {
                total_mileage += r.day_mileage;
                r.avg_oil = (r.day_oil * 100 / r.day_mileage).toFixed(2);
            }
        });
        return res.json({
            status : 'success', data: result, total_oil: total_oil, total_mileage: total_mileage,
            avg_oil: total_mileage > 0 ? (total_oil * 100 / total_mileage).toFixed(2) : 0
        });
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

// 速度段统计
m.speedDistribution = function (req, res) {
    var params  = ['start', 'end'];
    var dateReg = /\d{4}-\d{1,2}-\d{1,2}/;
    var sql     = 'select * from t_obd_drive where obdCode = ? and fireTime between ? and ?';

    if (params.find(e => !req.query[e])) {
        return res.json({status: 'failed', errorNo: 'INCOMPLETE_PARAMS', error: '请输入起止时间'});
    } else if (!dateReg.test(req.query.start) || !dateReg.test(req.query.end)) {
        return res.json({status: 'failed', errorNo: 'ILLEGAL_VALUE', error: '参数格式错误:起止时间参数start,end的格式应为"yyyy-MM-dd"'});
    }

    if (moment(req.query.start).add(100, 'days').isBefore(moment(req.query.end))) {
        return res.json({status: 'failed', errorNo: 'ERROR_PARAM', error: '起止时间间隔不能超过100天, 且开始时间应小于结束时间'});
    }

    co(function*() {
        var result      = yield mysql.pool.queryP(sql, [req.incar.vehicle.obd_code, req.query.start, req.query.end]);
        var speedLevels = [{speed: 1, time: 0, distance: 0},
            {speed: 45, time: 0, distance: 0},
            {speed: 90, time: 0, distance: 0},
            {speed: 255, time: 0, distance: 0}];
        result.forEach(r => {
            if (r.speedGroup) {
                var d = JSON.parse(r.speedGroup);
                d && d.forEach(i => {
                    var t     = i.speed;
                    var index = t == 1 ? 0 : t == 45 ? 1 : t == 90 ? 2 : t == 255 ? 3 : -1;
                    if (index < 0) return;
                    speedLevels[index].time += i.time;
                    speedLevels[index].distance += i.distance;
                });
            }
        });
        res.json({status: 'success', data: speedLevels});
    }).catch(ex => {
        res.status(500).json({status: 'failed', errorNo: 'SYSTEM_EXCEPTION', error: ex.message});
        console.log(__filename, ex.stack);
    });
};

module.exports = m;
