import {Injector} from 'ngES6';

export default class AppCtrl extends Injector {
    static $inject = ['$scope', '$state', '$http', '$timeout', 'AppService'];
    static $template = require('./app.html');

    constructor(...args) {
        super(...args);
        const {$scope} = this.$injected;
        this.attachMethodsTo($scope);
        $scope.form = {};
        $scope.pager = {page: 1, totalItems: 0, pageSize: 15};
        this.getApps();
    }

    getApps() {
        const {$scope, AppService} = this.$injected;
        const {page, pageSize} = $scope.pager;
        AppService.getApps(page, pageSize).then(res => {
            const result = this.checkResult(res);
            if (result) {
                $scope.apps = result.items;
                $scope.totalItems = result.total;
            }
        });
    }

    getDetail(appId) {
        const {$scope, AppService} = this.$injected;
        AppService.getApp(appId).then(res => {
            const result = this.checkResult(res);
            if (result) {
                $scope.app = result.app;
            }
        });
    }

    addApp() {
        const {$scope, AppService} = this.$injected;
        AppService.addApp($scope.form).then(res => {
            const result = this.checkResult(res);
            if (result) {
                $scope.form = {};
                if ($scope.pager.page === 1) {
                    $scope.apps.unshift(result.app);
                }
            }
        });
    }

    checkResult(res) {
        const {$scope} = this.$injected;
        if (res.data) {
            if (res.data.status === 'success') return res.data;
            $scope.notice = res.data.error;
        } else $scope.notice = '访问网络失败';

        return null;
    }
}
