import {Injector} from 'ngES6';
import $ from 'jquery';

export default class AppDetailCtrl extends Injector {
    static $inject = ['$scope', '$stateParams', '$timeout', '$http', 'AppService'];
    static $template = require('./detail.html');

    constructor(...args) {
        super(...args);
        const {$scope, $stateParams, AppService} = this.$injected;
        this.attachMethodsTo($scope);
        $scope.pager = {page: 1, totalItems: 0, pageSize: 15};

        AppService.getApp($stateParams.short_name).then(res => {
            let result = this.checkResult(res);
            if (result) {
                $scope.app = result.app;
                $scope.form = {short_name: $scope.app.short_name};
                this.getVersions();
            }
        });
    }

    getVersions() {
        const {$scope, AppService} = this.$injected;
        const {page, pageSize} = $scope.pager;
        AppService.getVersions($scope.app.short_name, page, pageSize).then(res => {
            let result = this.checkResult(res);
            if (result) {
                $scope.versions = result.items;
                $scope.pager.totalItems = result.total;
            }
        });
    }

    addVersion() {
        const {$scope, AppService} = this.$injected;
        if (!$scope.form.file_size) {
            $scope.notice = '请上传App文件!';
            return;
        }
        AppService.addVersion($scope.form).then(res => {
            const result = this.checkResult(res);
            if (result) {
                $scope.form = {short_name: $scope.app.short_name};
            }
        });
    }

    upload(event) {
        const {$scope, $http, $timeout, AppService} = this.$injected;
        let file  = $(event.target).find('input:file');

        $timeout(() => {
            file.click();
        });

        file.on('change', () => {
            if (file[0].files.length) {
                let form = new FormData();
                form.append('app', file[0].files[0]);
                $http({
                    url: AppService.uploadAppUrl,
                    method: 'POST',
                    headers: {'Content-Type': undefined},
                    data: form,
                    transformRequest: (data) => data,
                }).then(res => {
                    $scope.uploading = false;
                    let result = this.checkResult(res);
                    if (result) {
                        $scope.form.file_path = result.path;
                        $scope.form.file_size = result.size;
                    }
                });
            }
            $scope.uploading = true;
        });
    }

    makeDownloadUrl(filename) {
        return this.$injected.AppService.makeDownloadAppUrl(filename);
    }

    checkResult(res) {
        const {$scope} = this.$injected;
        if (res.data) {
            if (res.data.status === 'success') return res.data;
            $scope.notice = res.data.error;
        } else $scope.notice = '访问网络失败';

        return null;
    }
}
