import 'babel-polyfill';
import angular from 'angular';
import 'angular-animate';
import 'angular-ui-router';
import 'angular-ui-bootstrap';
import 'angular-chart.js';
import moment from 'moment';

import 'angular-bootstrap-datetimepicker';
import 'angular-bootstrap-datetimepicker/src/js/datetimepicker.templates.js';
import 'angular-bootstrap-datetimepicker/src/css/datetimepicker.css';
import 'bootstrap/dist/css/bootstrap.css';
import 'angular-chart.js/dist/angular-chart.css';
import './style/common.scss';

import directives from './directive';
import controllers, {templateCache} from './controller';
import services from './service';
import filters from './filter';
import config from './config';

moment.locale('zh-cn');
export const App = 'app';

const app = angular.module(App, ['ngAnimate', 'ui.router', 'ui.bootstrap',
    'chart.js', 'ui.bootstrap.datetimepicker',
    directives, services, controllers, filters])
    .provider('template', function() {
        this.$get = () => {return templateCache;};
    })
    .service('Settings', ['$location', function($location) {
        return {
            domain: $location.protocol() + '://' + $location.host() + ':' + $location.port() + '/wisecar',
            // domain: $location.protocol() + '://' + $location.host() + ':8000/wisecar',
            baiduMapKey: 'FPzmlgz02SERkbPsRyGOiGfj',
        };
    }])
    .run(['$rootScope', '$state', '$timeout', 'VehicleService', async function($rootScope, $state, $timeout, VehicleService) {
        $rootScope.$state = $state;
        const brands = await VehicleService.getBrands();
        $timeout(() => {
            $rootScope.brands = brands;
        });

        $rootScope.getSeries = async function(brandCode) {
            if (!$rootScope.series) $rootScope.series = {};
            if (!$rootScope.series[brandCode]) {
                $rootScope.series[brandCode] = [];
                const series = await VehicleService.getSeries(brandCode);
                $timeout(() => {
                    $rootScope.series[brandCode] = series;
                });
                return series;
            }
            return $rootScope.series[brandCode];
        };
    }]);

config.forEach(c => app.config(c));

window.name = 'NG_DEFER_BOOTSTRAP!';
angular.element().ready(() => {
    angular.resumeBootstrap([App]);
});

export default app;
