import {Injector} from 'ngES6';

export default class AppService extends Injector {
    static $inject = ['$http', 'Settings'];

    get uploadAppUrl() {
        return this.$injected.Settings.domain + '/api/web/app/upload';
    }

    makeDownloadAppUrl(filename) {
        return this.$injected.Settings.domain + '/api/web/app/download?filename=' + filename;
    }

    getApps(page, pageSize) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + '/api/web/app', {
            params: {pageSize: pageSize, nextMarker: (page - 1) * pageSize + 1},
        });
    }

    getApp(appId) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/app/${appId}`);
    }

    getVersions(shortName, page, pageSize) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/app/${shortName}/versions`, {
            params: {page: page, nextMarker: (page - 1) * pageSize + 1},
        });
    }

    getLatest(shortName) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/app/latest`, {
            params: {name: shortName},
        });
    }

    addApp(app) {
        const {$http, Settings} = this.$injected;
        return $http.post(Settings.domain + '/api/web/app', app);
    }

    addVersion(version) {
        const {$http, Settings} = this.$injected;
        return $http.post(Settings.domain + '/api/web/app/version', version);
    }
}
