import {Injector} from 'ngES6';

export default class CareService extends Injector {
    static $inject = ['$http', 'Settings'];

    async getConfig() {
        const {$http, Settings} = this.$injected;
        if (!this.careConfig) {
            await $http.get(Settings.domain + '/api/web/care/config').then(res => {
                if (res.data) {
                    if (res.data.status === 'success') this.careConfig = res.data.careConfig;
                }
            });
        }

        return this.careConfig;
    }

    getCares(vehicleId, page, pageSize) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/vehicle/${vehicleId}/care`, {
            params: {pageSize: pageSize, nextMarker: (page - 1) * pageSize + 1},
        });
    }

    addCare(vehicleId, care) {
        const {$http, Settings} = this.$injected;
        return $http.post(Settings.domain + `/api/web/vehicle/${vehicleId}/care`, care);
    }

    updateCare(careId, care) {
        const {$http, Settings} = this.$injected;
        return $http.put(Settings.domain + `/api/web/care/${careId}`, care);
    }

    deleteCare(careId) {
        const {$http, Settings} = this.$injected;
        return $http.delete(Settings.domain + `/api/web/care/${careId}`);
    }

    getCareStatus(vehicleId) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/vehicle/${vehicleId}/care/status`);
    }

    getCareAdvice(vehicleId) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/vehicle/${vehicleId}/care/advice`);
    }
}
