import {Injector} from 'ngES6';

export default class VehicleService extends Injector {
    static $inject = ['$http', 'Settings'];

    async getBrands() {
        const {$http, Settings} = this.$injected;
        if (!this.brands) {
            await $http.get(Settings.domain + '/api/pub/brand').then(res => {
                if (res.data) this.brands = res.data.brands;
            });
        }
        return this.brands;
    }

    async getSeries(brandCode) {
        const {$http, Settings} = this.$injected;
        if (!this.series) this.series = {};
        if (!this.series[brandCode]) {
            await $http.get(Settings.domain + `/api/pub/brand/${brandCode}/series`).then(res => {
                if (res.data) this.series[brandCode] = res.data.series;
            });
        }
        return this.series[brandCode];
    }

    getDetail(id) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/vehicle/${id}`);
    }

    getOilUsage(id, start, end) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/vehicle/${id}/oil`,
            {params: {start: start, end: end}});
    }

    getDrivingHabit(id, start, end) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/vehicle/${id}/habit`,
            {params: {start: start, end: end}});
    }

    getSpeedDistribution(id, start, end) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/vehicle/${id}/speed`,
            {params: {start: start, end: end}});
    }

    getTrips(id, page, pageSize) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/vehicle/${id}/trip`,
            {params: {nextMarker: (page - 1) * pageSize + 1, pageSize: pageSize}});
    }

    getTracks(id, page, pageSize) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/vehicle/${id}/track`,
            {params: {pageSize: pageSize, nextMarker: (page - 1) * pageSize + 1}});
    }

    getReports(id, type, page, pageSize) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/vehicle/${id}/report`,
            {params: {type: type, pageSize: pageSize, nextMarker: (page - 1) * pageSize + 1}});
    }

    getDetects(id, page, pageSize) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/vehicle/${id}/detect`,
            {params: {pageSize: pageSize, nextMarker: (page - 1) * pageSize + 1}});
    }

    getCares(id, page, pageSize) {
        const {$http, Settings} = this.$injected;
        return $http.get(Settings.domain + `/api/web/vehicle/${id}/care`,
            {params: {pageSize: pageSize, nextMarker: (page - 1) * pageSize + 1}});
    }
}
